/*
  GtkDiskFree shows free space on your mounted partitions.  Copyright
  (C) 2001-2002 Dj-Death (Landwerlin Lionel)
  (C) 2012-2016 mazes_80 (Bauer Samuel)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  USA */

#include "pixmap.h"
#include "configure.h"

#ifdef HAVE_MATH_H
# include <math.h>
#endif

extern GtkWidget *mwindow;

/* TODO Find a better way to factorise code */
/* TODO Make proper interaction between cairo and gdk */
/* This is an ugly workaround */
void create_and_fill_pixbuf(gboolean freecairo, GtkWidget *top, const gdouble *color, GdkPixbuf **pixbuf, int width, int height, cairo_surface_t **surface, cairo_t **pcr) {
    gboolean done = FALSE;
    cairo_surface_t *sr;
    cairo_t *cr;

    if(*pixbuf == NULL)
        *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);

    if(freecairo) {
        pcr = &cr;
        surface = &sr;
    }

    *surface = cairo_image_surface_create_for_data( gdk_pixbuf_get_pixels(*pixbuf),
                                CAIRO_FORMAT_ARGB32, width, height,
                                gdk_pixbuf_get_rowstride(*pixbuf));
    *pcr = cairo_create(*surface);

#if GTK_CHECK_VERSION(3, 0, 0)
    if(top) {
        GtkStyleContext *ctx = NULL;
        ctx = gtk_widget_get_style_context (top);
        gtk_render_background( ctx, *pcr, 0.0, 0.0, width - 1.0, height - 1.0);

        done = TRUE;
    }
#endif
    if(!done) {
        if(color)
            cairo_set_source_rgb(*pcr, color[2], color[1], color[0]);
        else
            cairo_set_source_rgba(*pcr, 0.0, 0.0, 0.0, 0.0);
        cairo_rectangle(*pcr, 0.0, 0.0, width, height);
        cairo_fill(*pcr);
    }

    if(freecairo) {
        /* Free the allocated structures. */
        cairo_surface_destroy(sr);
        cairo_destroy(cr);
    }
}

void fill_pixbuf_color(const gdouble *color, GdkPixbuf **pixbuf, int width, int height) {
    create_and_fill_pixbuf(TRUE, NULL, color, pixbuf, width, height, NULL, NULL);
}

GdkPixbuf *pixmap_draw_capacity (gint real, gint mounted, gint usedp, gint width)
{
    GdkPixbuf *pixbuf = NULL;
    double x, y, max;
    gint height = 12;
    gchar *text;

    cairo_text_extents_t extents;
    cairo_surface_t *surface;
    cairo_pattern_t *pattern;
    cairo_t *cr;

    max = width / 100.0;
    max = max * usedp;

    create_and_fill_pixbuf(FALSE, NULL, NULL, &pixbuf, width, height, &surface, &cr);

    /* Foreground color. */
    if (!real || mounted == FS_UMOUNTED) {
        cairo_set_source_rgb(cr, options->color3[2], options->color3[1], options->color3[0]);
        cairo_rectangle(cr, 0, 0, width, height);
        cairo_fill(cr);
    } else if (options->capacity_style[0]) {
        pattern = cairo_pattern_create_linear(0.0, 0.0, width, 0.0);
        cairo_pattern_add_color_stop_rgba(pattern, 0, options->color1[2], options->color1[1], options->color1[0], 1);
        cairo_pattern_add_color_stop_rgba(pattern, 1, options->color2[2], options->color2[1], options->color2[0], 1);
        cairo_rectangle(cr, 0, 0, max, height - 1);
        cairo_set_source(cr, pattern);
        cairo_fill(cr);
        cairo_pattern_destroy(pattern);
    } else {
        gdouble faded_color[3];
        int idx;
        for(idx=0; idx < 3; idx++)
            faded_color[idx] = ( options->color1[idx] * (gdouble)(100 - usedp) ) / 100.0 + ( options->color2[idx] * (gdouble)usedp ) / 100.0;
        cairo_set_source_rgb(cr, faded_color[2], faded_color[1], faded_color[0]);

        cairo_rectangle(cr, 1, 1, max, height - 1);
        cairo_fill(cr);
    }

    /* Add text in capacity column ? */
    if (options->capacity_style[1] && mounted == FS_MOUNTED && real) {
        text = g_strdup_printf("%i %%", usedp);
        cairo_set_source_rgb(cr, options->color_text[2], options->color_text[1], options->color_text[0]);
        cairo_set_font_size(cr, 8);
        cairo_text_extents(cr, text, &extents);
        x = width / 2 - (extents.width / 2 + extents.x_bearing);
        y = height / 2 - (extents.height / 2 + extents.y_bearing);
        cairo_move_to(cr, x, y);
        cairo_show_text(cr, text);
        g_free(text);
    }

    cairo_rectangle(cr, 0, 0, width, height);
    cairo_set_source_rgb(cr, 0, 0, 0);
    cairo_set_line_width(cr, 1);
    cairo_stroke(cr);

    /* Free the allocated structures. */
    cairo_surface_destroy(surface);
    cairo_destroy(cr);

    return pixbuf;
}

void legend_draw(cairo_t *cr, const gchar *text, const gdouble *color, const double x, const double y)
{
    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
    cairo_rectangle(cr, x, y, 10.0, 10.0);
    cairo_stroke(cr);
    cairo_set_source_rgb(cr, color[2], color[1], color[0]);
    cairo_rectangle(cr, x, y, 10.0, 10.0);
    cairo_fill(cr);
    cairo_set_source_rgb(cr, options->color_text[2], options->color_text[1], options->color_text[0]);
    cairo_move_to(cr, x + 16.0, y + 8.0);
    cairo_show_text(cr, text);
}

GdkPixbuf *pixmap_draw_legend(GtkWidget *top, gint width, gint height) {
    GdkPixbuf *pixbuf = NULL;
    cairo_surface_t *surface;
    cairo_t *cr;

    create_and_fill_pixbuf(FALSE, top, NULL, &pixbuf, width, height, &surface, &cr);

    cairo_set_line_width(cr, 1.0);

    legend_draw(cr, _("Free"), options->color1, 8.0, 2.0);
    legend_draw(cr, _("Used"), options->color2, (double)width / 2.0 + 8.0, 2.0);

    /* Free the allocated structures. */
    cairo_surface_destroy(surface);
    cairo_destroy(cr);

    return pixbuf;
}

GdkPixbuf *pixmap_draw_fs_snail_chart(GtkWidget *top, gint size, gint total, gint ufree, gint sfree) {
    GdkPixbuf *pixbuf = NULL;
    cairo_surface_t *surface;
    cairo_t *cr;

    double angle = 6.0 * M_PI + M_PI_2;
    double step = 0.05 * M_PI;
    double radius = (double)size / 6.0;
    double r_step = radius / 100.0;
    double center = ((double)(size)) * 0.5;
    double line_size = (double)size / 100.0;
    double w_color[3]; /* Weighted limit color */
    double i_color[3]; /* Interpolated color */
    double idx_d;
    double ipol;
    double x, y;
    gint limit;
    int idx, i;

    idx_d = (double)(total - ufree - sfree) / (double)total;

    if(options->popup[2])
        for(idx = 0; idx < 3; idx++)
            w_color[idx] = (1.0 - idx_d) * options->color1[idx] + idx_d * options->color2[idx];

    limit = 100.0 * idx_d;

    create_and_fill_pixbuf(FALSE, top, NULL, &pixbuf, size, size + (double)size * 0.1, &surface, &cr);

    cairo_set_line_width(cr, line_size);
    for(idx=100, idx_d=1.0; idx > 0; idx--) {
        x = center * ( 1.0 + 0.6 * idx_d * cos(angle) );
        y = center * ( 1.0 + 0.6 * idx_d * sin(angle) );

        cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
        cairo_move_to(cr, x, y);
        cairo_arc (cr, x, y, radius, 0, 2.0 * M_PI);
        cairo_stroke_preserve(cr);

        /* Interpolate_color */
        /* TODO refactor */
        if(idx > limit) {
            if(options->popup[2]) {
                /* progression in free range */
                ipol = (double)(idx - limit)/(double)(100 - limit);
                ipol = 1 - ipol;
                ipol = ipol * ipol;
                for(i = 0; i < 3; i++)
                    i_color[i] = ipol * w_color[i] + (1.0 - ipol) * options->color1[i];
                cairo_set_source_rgb(cr, i_color[2], i_color[1], i_color[0]);
            }
            else
                cairo_set_source_rgb(cr, options->color1[2], options->color1[1], options->color1[0]);
        }
        else {
            if(options->popup[2]) {
                /* progression in used range */
                ipol = (double)(limit - idx)/(double)(limit);
                ipol = 1 - ipol;
                ipol = ipol * ipol;
                ipol = ipol * ipol;
                for(i = 0; i < 3; i++)
                    i_color[i] = ipol * w_color[i] + (1.0 - ipol) * options->color2[i];
                cairo_set_source_rgb(cr, i_color[2], i_color[1], i_color[0]);
            }
            else
                cairo_set_source_rgb(cr, options->color2[2], options->color2[1], options->color2[0]);
        }
        cairo_move_to(cr, x, y);
        cairo_arc (cr, x, y, radius, 0, 2.0 * M_PI);
        cairo_fill(cr);

        if(idx % 2 == 0 || idx % 3 == 0)
            radius = radius - r_step;
        if(idx % 10 == 0)
            r_step = r_step + r_step / 25.0;
        idx_d = idx_d - 0.01;
        angle = angle - step;
        if(idx % 6 == 0) {
            line_size = line_size - line_size / 12.5 - line_size / 25.0;
            cairo_set_line_width(cr, line_size);
        }
    }

    /* Free the allocated structures. */
    cairo_surface_destroy(surface);
    cairo_destroy(cr);

    return pixbuf;
}

void pie_chart_draw_slice(cairo_t *cr, const gchar *legend, gdouble *color, double angle1, double angle2, double size) {
    cairo_text_extents_t extents;
    double center = size * 0.5;
    double radius = center * 0.95;
    double x, y;

    cairo_set_source_rgb(cr, color[2], color[1], color[0]);
    cairo_move_to(cr, center, center);
    cairo_arc (cr, center, center, radius, angle1, angle2);
    cairo_fill(cr);
    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
    cairo_text_extents(cr, legend, &extents);
    x = center * ( 1.0 + 0.6 * cos(angle2/2.0) );
    y = center * ( 1.0 + 0.6 * sin(angle2/2.0) );
    x = x - (extents.width / 2.0 + extents.x_bearing);
    y = y - (extents.height / 2.0 + extents.y_bearing);
    cairo_move_to(cr, x, y);
    cairo_show_text(cr, legend);
    cairo_move_to(cr, center, center);
    cairo_arc (cr, center, center, radius, angle1, angle2);
    cairo_set_line_width(cr, 1);
    cairo_stroke(cr);
}

GdkPixbuf *pixmap_draw_fs_pie_chart(GtkWidget *top, gint size, gint total, gint ufree, gint sfree) {
    GdkPixbuf *pixbuf = NULL;
    double angle;

    cairo_surface_t *surface;
    cairo_t *cr;

    create_and_fill_pixbuf(FALSE, top, NULL, &pixbuf, size, size, &surface, &cr);

    angle = (2 * M_PI) * (double)(total - ufree - sfree) / (double)total;
    pie_chart_draw_slice(cr, _("Used"), options->color2, 0, angle, (double)size);
    pie_chart_draw_slice(cr, _("Free"), options->color1, angle, 2 * M_PI, (double)size);

    /* Free the allocated structures. */
    cairo_surface_destroy(surface);
    cairo_destroy(cr);

    return pixbuf;
}

GdkPixbuf *pixmap_draw_mount(GtkWidget *top, gint size, gdouble *color) {
    GdkPixbuf *pixbuf = NULL;
    cairo_pattern_t *pat;
    cairo_surface_t *sr;
    cairo_t *cr;

    double x, y;
    double radius;

    create_and_fill_pixbuf(FALSE, NULL, NULL, &pixbuf, size, size, &sr, &cr);

    x = (double)size * 0.5;
    y = x - (double)size * 0.1925;
    radius = x * 0.95;

    pat = cairo_pattern_create_radial (x , x, 0.65, y, y, radius);
    cairo_pattern_add_color_stop_rgba (pat, 0.0, 1.0, 1.0, 1.0, 1.0);
    cairo_pattern_add_color_stop_rgba (pat, 0, color[2], color[1], color[0], 1);
    cairo_pattern_add_color_stop_rgba (pat, 1, 0, 0, 0, 1);
    cairo_set_source (cr, pat);
    cairo_arc (cr, x, x, radius, 0, 2 * M_PI);
    cairo_fill (cr);

    /* Free the allocated structures. */
    cairo_pattern_destroy(pat);
    cairo_surface_destroy(sr);
    cairo_destroy(cr);

    return pixbuf;
}

